use viperid::VResult;
use wasmi::{Engine, Linker, Module, Store};

use crate::wasi::{self, StockWasi};

struct ExecutorState {
    wasi: wasi::StockWasi
}

impl ExecutorState {
    fn new() -> Self {
        ExecutorState {
            wasi: StockWasi::new()
        }

    }
}

pub fn run_entry_point(wasm: &[u8]) -> VResult<()> {
    let engine = Engine::default();

    let module = Module::new(&engine, wasm)?;

    let mut store = Store::new(&engine, ExecutorState::new());

    let mut linker = <Linker<ExecutorState>>::new(&engine);
    wasi::integrate(&mut linker, |hs| &mut hs.wasi)?;

    let instance = linker
        .instantiate(&mut store, &module)?
        .start(&mut store)?;

    let main = instance.get_typed_func::<(), ()>(&store, "_start")?;
    main.call(&mut store, ())?;

    Ok(())
}